// Versuch 2_2
// C. Roppel, HS Schmalkalden
// MATLAB- und Scilab-Übungen zum Buch "Grundlagen der Nachrichtentechnik", Hanser-Verlag, 2018
// Getestet mit Scilab 5.5.2
mode(0);
ieee(1);
clear
clc
xdel(winsid())

//**********
// Parameter
//**********
alpha = 0.5;     // Roll-off-Faktor
os = 8;          // Samples pro Symbol (Oversampling Ratio)
dly_rc = 3;      // Verzögerung des Wurzel-Kosinus-roll-off-Filters
nSym = 1000;     // Anzahl Symbole
m = 4;           // m-wertige Symbole
k = log2(m);     // k Bit pro Symbol
lambda = %pi/4;  // QPSK Phasenoffset
EbN0 = 6;        // Eb/N0 in dB

//*******************************
// Wurzel-Kosinus-roll-off-Filter
//*******************************
t = -dly_rc*os:dly_rc*os;

psrc = (4*alpha*(t/os).*cos(%pi*(1 + alpha)*(t/os)) + sin(%pi*(1 - alpha)*(t/os)))./..
((1 - (4*alpha*(t/os)).^2)*%pi.*(t/os));
// Grenzwert für t/T_s = 0
psrc(dly_rc*os + 1) = (4*alpha + %pi*(1 - alpha))/%pi;
// Grenzwert für t/T_s = 1/(4 alpha))
if round(os/(4*alpha)) == os/(4*alpha) then  // Ist os/(4*alpha) ganzzahlig?
    psrc(dly_rc*os + 1 + os/(4*alpha)) = (4*alpha*cos(%pi*(1 + alpha)/(4*alpha)) - %pi*(1 + alpha)..
    *sin(%pi*(1 + alpha)/(4*alpha)) + %pi*(1 - alpha)*cos(%pi*(1 - alpha)/(4*alpha)))/(-2*%pi);
    psrc(dly_rc*os + 1 - os/(4*alpha)) = psrc(dly_rc*os + 1 + os/(4*alpha));
end

E = sum(psrc.^2);  // Energie
psrc = psrc/sqrt(E);  // Skalierung auf E = 1

//**********************************
// QPSK-Modulator und Pulsformfilter
//**********************************
grand("setsd", 1234);  // Initialisiere Zufallsgenerator
symbols = grand(nSym, 1, "uin", 0, m - 1);  // Spaltenvektor der Länge nSym mit Zufallszahlen im Bereich 0 bis m-1

// Modulation
xmod = exp(%i*((2*%pi/m).*symbols + lambda));

xmod = upsample(xmod, os);  // Einfügen von os - 1 Nullen zwischen den Symbolen
x = conv(psrc, xmod);  // Faltung (convolution)
x = x(dly_rc*os + 1:$ - dly_rc*os);  // Entfernen der ersten und letzten Werte

//***********
// AWGN-Kanal
//***********
snr = EbN0 + 10*log10(k) - 10*log10(os);  // Störabstand in dB
sigPwr = 0.5*mean(abs(x).^2);  // Signalleistung
noisePwr = sigPwr/(10^(snr/10));  // Rauschleistung
rand("seed", 1234);
noise_i = rand(length(x), 1, "normal")*sqrt(noisePwr);
rand("seed", 4321);
noise_q = rand(length(x), 1, "normal")*sqrt(noisePwr);
y = x + noise_i + %i*noise_q;  // Signal + Rauschen

// Leistungsdichtespektrum (Periodogramm)
len = length(y);
len = 2*floor(len/2);
y = y(1:len);
f = (os*(-len/2:len/2 - 1)/len)';
lds = 10*log10((1/(len*os))*fftshift(abs(fft(y)).^2));
figure;
plot(f, lds)
xlabel("f in Hz");
ylabel("LDS in dB");

//**********************************************
// Signalangepasstes Filter und QPSK-Demodulator
//**********************************************
ye = conv(psrc, y);
ye = ye(dly_rc*os + 1:$ - dly_rc*os);

ye = downsample(ye, os);

// Scatterplot
figure;
plot(real(ye), imag(ye), 'ob')
xlabel("y_i"); 
ylabel("y_q");

// Demodulation
ye = ye.*exp(-%i*lambda);
ysym = round(m/(2*%pi)*atan(imag(ye), real(ye)));
ysym(ysym < 0) = ysym(ysym < 0) + m;

// Bestimmung der Symbolfehlerhäufigkeit
nerror = sum(ceil(bitxor(symbols, ysym)/m));
ser = nerror/length(ysym);
serTheo = erfc(sqrt(10^(EbN0/10)));
mprintf('Eb/N0: %f dB  Ps: %f  Symbolfehler: %d  Symbolfehlerhäufigkeit: %f\n', EbN0, serTheo, nerror, ser);
