// Versuch 3_1
// C. Roppel, HS Schmalkalden
// MATLAB- und Scilab-Übungen zum Buch "Grundlagen der Nachrichtentechnik", Hanser-Verlag, 2018
// Getestet mit Scilab 5.5.2
mode(0);
ieee(1);
clear
clc
xdel(winsid())

//**********
// Parameter
//**********
alpha = 0.5;     // Roll-off-Faktor
os = 8;          // Samples pro Symbol (Oversampling Ratio)
dly_rc = 3;      // Verzögerung des Wurzel-Kosinus-roll-off-Filters
nSym = 1000;     // Anzahl Symbole
m = 4;           // m-wertige Symbole
k = log2(m);     // k Bit pro Symbol
lambda = %pi/4;  // QPSK Phasenoffset

//*******************************
// Wurzel-Kosinus-roll-off-Filter
//*******************************
t = -dly_rc*os:dly_rc*os;

psrc = (4*alpha*(t/os).*cos(%pi*(1 + alpha)*(t/os)) + sin(%pi*(1 - alpha)*(t/os)))./..
((1 - (4*alpha*(t/os)).^2)*%pi.*(t/os));
// Grenzwert für t/T_s = 0
psrc(dly_rc*os + 1) = (4*alpha + %pi*(1 - alpha))/%pi;
// Grenzwert für t/T_s = 1/(4 alpha))
if round(os/(4*alpha)) == os/(4*alpha) then  // Ist os/(4*alpha) ganzzahlig?
    psrc(dly_rc*os + 1 + os/(4*alpha)) = (4*alpha*cos(%pi*(1 + alpha)/(4*alpha)) - %pi*(1 + alpha)..
    *sin(%pi*(1 + alpha)/(4*alpha)) + %pi*(1 - alpha)*cos(%pi*(1 - alpha)/(4*alpha)))/(-2*%pi);
    psrc(dly_rc*os + 1 - os/(4*alpha)) = psrc(dly_rc*os + 1 + os/(4*alpha));
end

E = sum(psrc.^2);  // Energie
psrc = psrc/sqrt(E);  // Skalierung auf E = 1

//**********************************
// QPSK-Modulator und Pulsformfilter
//**********************************
grand("setsd", 1234);  // Initialisiere Zufallsgenerator
symbols = grand(nSym, 1, "uin", 0, m - 1);  // Spaltenvektor der Länge nSym mit Zufallszahlen im Bereich 0 bis m-1

// Modulation
xmod = exp(%i*((2*%pi/m).*symbols + lambda));

xmod = upsample(xmod, os);  // Einfügen von os - 1 Nullen zwischen den Symbolen
x = conv(psrc, xmod);  // Faltung (convolution)
x = x(dly_rc*os + 1:$ - dly_rc*os);  // Entfernen der ersten und letzten Werte

//**************
// Mehrwegekanal
//**************
dt = 2;   // Laufzeitdifferenz (in Samples)
a = 0.8;  // Dämpfung des indirekten Pfades
b = 1;    // Produkt f_c * t_0

// Impulsantwort
hKanal = zeros(1, dt + 1);
hKanal(1) = 1;
hKanal(dt + 1) = a*exp(-%i*2*%pi*b);

//Übertragungsfunktion
f = (os*(-256:255)/512);
HKanal = 1 + a*exp(-%i*2*%pi*b)*exp(-%i*2*%pi*f*dt/os);
figure;
plot(f, abs(HKanal));
xlabel('f in Hz');
ylabel('|H_K(f)|');

// Faltung mit der Kanalimpulsantwort
r = conv(hKanal, x);

// Leistungsdichtespektrum (Periodogramm)
len = length(r);
len = 2*floor(len/2);
r = r(1:len);
f = (os*(-len/2:len/2 - 1)/len)';
lds = 10*log10((1/(len*os))*fftshift(abs(fft(r)).^2));
figure;
plot(f, lds)
xlabel("f in Hz");
ylabel("LDS in dB");

//*************************
// Signalangepasstes Filter
//*************************
ye = conv(psrc, r);
ye = ye(dly_rc*os + 1:$ - dly_rc*os);

// Augendiagramm
yi = real(ye);
yq = imag(ye);
eyemax = floor(length(yi)/(2*os));
teye = ((-os:os)/os)';
figure;
subplot(211);
for i = 1:eyemax - 1
    plot(teye, yi(2*(i - 1)*os + 1:2*i*os + 1));
end
plot(teye(1:$-1), yi(2*(eyemax - 1)*os + 1:2*eyemax*os));
xlabel("t/T_s");
ylabel("y_i");
subplot(212);
for i = 1:eyemax - 1
    plot(teye, yq(2*(i - 1)*os + 1:2*i*os + 1));
end
plot(teye(1:$-1), yq(2*(eyemax - 1)*os + 1:2*eyemax*os));
xlabel("t/T_s");
ylabel("y_q");
